-- #########################################################

                     AldorAST.lhs

                  Chris Ryder, 1/7/99

Types for representing Aldor abstract syntax trees (AST's)
in Haskell programs. To be used as a "library" in
combination with the Haskell output of the Aldor compiler
(using the -Fhs option)

-- #########################################################

> module AldorAST where

Uses a single type to represent the tree. This almost a
straight C -> Haskell conversion of the types used in the
Aldor compiler.

When "Symbol" type is found in the C code, "String"s have
been used instead. May have to make a different Haskell type
for Symbols, etc, if more information is needed in the Haskell
representation.

This type has 73 differnt cases.

> data AST =

First, some nodes will not immediatly be implemented, so an
"error" node is included.

>	Error String |

A tree can of course be empty

>	Null |

----------------------
----- Leaf Nodes -----
----------------------

The various Literals in the AST

>	LitInt    Integer |
>	LitFloat  Double  |
>	LitString String  |

Identifiers. The String is the symbol. Perhaps need symbol
type ????

>	Ident String |

This is the abIdSys node type. Is this somekind of "system
identifier ?" - Could be for "0" and "1" which are special
cases ?

>	IdentSy String |

Not sure what this node is for. Put it in to ease translation
from Aldor AST -> Haskell representation.

>	Blank String |

Not sure if this node type is actually used in the Aldor
compiler, but could be interesting if it is. Not sure if
String is the right type here.

>	DocText String |

--------------------------
----- Interior Nodes -----
--------------------------

This is the Add node for the keyword "add". Used in type
or domain definitions in Aldor. Format is :
  parent-domain add declarations

>	Add AST AST |

And nodes. Used in logical expressions.

>	And [AST] |

This is for the application of operators to arguments. The
first AST is the operator. The list of AST's is the arguments.

>	Apply [AST] |

Assert nodes. For #assert identifier ?? This is the only
"assert" I can find in the manual. In the C structure, the
AST field is labeled "test". In the Aldor manual, the format
is #assert idetifier.

>	Assert AST |

This is for the assignment of values to variables. The two
AST's are the left and right sides of the assignment.

>	Assign AST AST |

Break node for breaks. The AST is the label to "break" to
(break in a nested loop only breaks out of the deepest loop).
Break can be viewed as a function with return type Exit.

>	Break AST |

Builtin node. Builtin is a type which can be used in
conjunction with the type "Foreign" to interface to abstract
machine level operations. (See Page 151 of Aldor manual).

>	Builtin AST |

Do node. The "do" expression takes one argument, an
expression, and evaluates it, discarding the result. Hence it
returns no value. In this Haskell type, AST is the expression.

>	Do AST |

Documented node. Not sure what this node type is for. Possibly
holds source code comments ?

>	Documented AST AST |

CoerceTo node. Used to convert between types. First AST is the
expression to coerce, the second is the type to coerce to.

>	CoerceTo AST AST |

Collect nodes. Used in generate expressions. The first AST is
the body of the generator. The list of AST's is the iteration
expressions. (See page 60 of manual)

>	Collect AST [AST] |

Comma nodes. A comma seperated list of experssions ??

>	Comma [AST] |

This is used for the definition of the declaration statements.
The first AST is the identifier, the second is the type

>	Declare AST AST |

Default nodes. Could be either a default modifer (page 113)
or default argument (page 67). Not sure which.

>	Default AST |

Define. Define modifier ? (page 113). AST's are left and
right hand sides.

>	Define AST AST |
>	DDefine AST |

Except nodes. Not sure of the purpose of this node type. First
AST is a type. The second is labeled "except" in the C code.

>	Except AST AST |

Exit nodes. Possible the Exit type. If this type is returned
by any function the program will exit (see page 150 of manual).
Not sure though, since the node has two subtrees labeled "test"
and "value" respectivly.

>	Exit AST AST |

Export nodes. Used for the export modifier (see page 114 of
manual). First AST is the thing to export, the second is the
"from" part and the third is the "to" part.

>	Export AST AST AST |

Extend nodes. For the extend keyword. AST has the label "body".
Not sure what it does though.

>	Extend AST |

Fix nodes. Not sure what this node does. Has one field, labeled
"function".

>	Fix AST |

Fluid nodes. For the "fluid" keyword. Used for giving
identifiers dynamic as opposed lexical scope. (see p114)
Not sure what the AST list is for (presumably the experssions
to be made fluid).

>	Fluid [AST] |

For nodes. Used in For loops. Format : for lhs in Expr | cond
The first AST in the node is the lhs, the second is the body
of the loop, the third is the condition / Expr.

>	For AST AST AST |

Foreign. Used for importing / exporting values that are not
produced by Axiom (eg, Lisp or C code). First AST hsa the
label "what" in the C code, the second has the label "origin".
Not sure what these hold.

>	Foreign AST AST |

Free. If this refers to the modifer "free", this is used to
bring a variable that is out of scope, into scope. Not sure
what the list of AST's is for. Presumably, one will be the
variable that is brought into scope.

>	Free [AST] |

Generate. Represents generators (similar to list comprehension,
see page 122 of manual). The first AST is labeled "count" in
the C code. The second is the body of the Generate expression.

>	Generate AST AST |

Goto. Goto a label. AST is the label. Is label a pointer
to some other part of the tree, or just an AST that describes
the label ?

>	Goto AST |

Has. Does this check to see if an expression has some
property ? The first AST is labeled "expr" the second is
labeled "property".

>	Has AST AST |

Hide. Dont know what this is for. Seems to accept an AST that
is a type.

>	Hide AST |

Hook. As for hide, not sure what this does. The AST is an
experssion.

>	Hook AST |

If. The If statement representation. The first AST is the
condition expression, the second is for the "then" expression,
and the third is for the "else" expression.

>	If AST AST AST |

Import. Import something from somewhere (see page 106 of
manual for details). The first AST is what to import, the
second is where to import it from.

>	Import AST AST |

Inline. Similar to Import, but allows compilation of current
scope to depend on compiled values of the constants indicated.
(For more information on the use of inline, see page 107 of
the manual). The type is exactly the same as Import.

>	Inline AST AST |

Iterate. Causes the current iteration of a loop to be
halted and the next iteration started. AST is a label. Does
this label just point to the part of the AST that represents
the loop ? Could cause problems when traversing tree if that
is the case ("cyclic" structure).

>	Iterate AST |

Label. Representation of a "label expression". The first AST
is the label, the second AST is the expression.

>	Label AST AST |

Lambda. Representation of a lambda expression. First AST is
the parameters, the second is the return type and the third
is the body of the function definition.

>	Lambda AST AST AST |

Let. Not sure what this is for. First AST has a label "context"
and the second a label "expr".
-- It's like in Haskell: "Let ... in ..." -- L.T.

>	Let AST AST |

Local. Declares a given identifier to be local to the current
scope. No type is specified (deduced at first assignment). An
initial assignment may also be included in the expression,
hence the list of AST's.

>	Local [AST] |

Macro. A representation of a macro. The AST is the expression.

>	Macro AST |

MDefine. A definition of a macro ?. The first AST is the left
hand side, the second is the right hand side.

>	MDefine AST AST |

MLambda. Lambda expression for macro ? First AST is the
parameters, the second is the body.

>	MLambda AST AST |

Never. A special value of the Exit type. An assertion (by
programmer) that execution will never reach the "never"
expression.

>	Never |

Not. The logical inverse of an expression. The AST is the
expression to be inverted.

>	Not AST |

Nothing. Not sure what this is for. Seems rather odd.
(Renamed ABNothing because of a name clash with a Haskell
identifier)

>	ABNothing |

Or. The logical OR of the expressions.

>	Or [AST] |

Paren. Parenthesis around an expression ?

>	Paren AST |

PLambda. Lambda expression for parenthesis ? First AST is
"param", second "rtype" third "body".

>	PLambda AST AST AST |

PretendTo. Primitive type conversion. "expr pretend type"
Causes expr to be treated as of type "type". The first AST is
the expression, the second is the type.

>	PretendTo AST AST |

Qualify. Used for qualifying (disambiguating) expressions.
The first AST is the expression, the second is what it should
be qualified as.

>	Qualify AST AST |

Raise. Not sure what this does. The AST is an expression.
Raises an exception, perhaps? -- L.T.

>	Raise AST |

Repeat. The representaion of a repeat expression loop. The
first AST is the body of the loop. The list of AST's is a
list of iterators - expressions to control the number of
times the loop body is evaluated.

>	Repeat AST [AST] |

RestrictTo. Not sure of the use of the use of this. The first
AST is an expression, the second is a type.

>	RestrictTo AST AST |

RetractTo. As for RestrictTo.

>	RetractTo AST AST |

Return. The return expression. AST is the expression.

>	Return AST |

Sequence. A sequence of expressions. Represented as a list of
AST's.

>	Sequence [AST] |

Test. Used in loops ? -- Also inside If! -- L.T. The AST is a condition
expression.

>	Test AST |

Try. Not sure what this does. First AST is labeled "expr",
second "id", third "except" and fouth "always".
Is this an exception-catching block with "finally"?-- L.T.

>	Try AST AST AST AST |

Where. Similar to "where" in Haskell/Miranda. The declarations
and definitons in the first AST are used in the evaluation of
the second AST.

>	Where AST AST |

While. Allows loop to continue as long as an expression is
true. AST is that expression.

>	While AST |

With. Used in defining categories (see P93 in manual). The
first AST is the expr the second is the body.

>	With AST AST |

Yield. Used in generators. Yield passes back a value and halts
the iteration of the generator. (until next request from it).
AST is the value returned.

>	Yield AST

That's all !..

>	deriving (Show, Read)

