-- vim:ts=4:syntax=haskell
-- ========================================================================= --
--								  "MkEnvs.hs":								 --
--																			 --
--	Here we put the functions building various Environments (e.g. Evaluation --
--	or Type-Checking). The Envs contain function objects (to work around the --
--	difficulties with mutually-recursive Haskell modules) which  are defined --
--	in the "earlier" modules (imported by this one).  The ctor functions can --
--	also use command-line parms in building the Envs.						 --
--																			 --
--							L.A.Timochouk, 2000--2002						 --
-- ========================================================================= --
module MkEnvs
	   (
			mkETC, CmdL_Arg (..)
	   )
where

import NameBinds
import EvalLib
import qualified EvalX
import qualified TypeCheckTER

--modName :: String
--modName =  "MkEnvs"

---------------------------------------
--	Defaults for some config values: --
---------------------------------------
defaultMaxEvalDepth :: Int
defaultMaxEvalDepth =  1000

defaultMaxADEDepth  :: Int
defaultMaxADEDepth  =  10


-------------------
--	"Cmdl_Arg":  --
-------------------
data  CmdL_Arg =
	  Debug_Flag
	| NoCache_Flag
	| Strong_EvalTC_Flag
	| Max_ADE_Depth  Int
	| Max_Eval_Depth Int
	| AST_File       String
	deriving (Eq)

----------------
--	"mkETC":  --
----------------
--	Creates an initial "ETC_Env" with empty stack, type inference set to False
--		(it will be turned on  selectively  by type-checking functions).  Some
--		other fields are set via the command-line parms:
--
mkETC :: NameBinds -> [CmdL_Arg] -> ETC_Env

mkETC    binds        pargs   =
	ETC_Env {_bindsET      = binds,
 	 		 _evalStack    = [],

    		 _remEvalDepth = maxEvalDepth, 
			 _maxADEpth    = maxADEpth,

			 _inferTypes   = False,
			 _noEval       = False,

			 _debugMode    = debugMode,
			 _strongEvalTC = strongEvalTC,
			 _useCaching   = not noCaching,

			 _evalFB	   = EvalX.evalFB,
			 _evalExpr	   = EvalX.evalExpr,
			 _evalDT	   = EvalX.evalDT,

			 _tcFB_T       = TypeCheckTER.typeCheckFB_T,
			 _tcExpr_T     = TypeCheckTER.typeCheckExpr_T,
			 _tcFB_E       = TypeCheckTER.typeCheckFB_E,
			 _tcFB_R       = TypeCheckTER.typeCheckFB_R
		   }
	where
	maxEvalDepth :: Int
	maxEvalDepth =  case [med | Max_Eval_Depth med  <- pargs] of
		[] ->
			--	Option not specified, use the default:
			defaultMaxEvalDepth
		yes->
			--	The last value (if several)  prevails:
			last yes

	maxADEpth    :: Int
	maxADEpth    =  case [made | Max_ADE_Depth made <- pargs] of
		[] ->
			--	Option not specified, use the default:
			defaultMaxADEDepth
		yes->
			--	The last value (if several)  prevails:
			last yes

	debugMode    :: Bool
	debugMode    =  [df | df@Debug_Flag 		 <- pargs] /= []

	strongEvalTC :: Bool
	strongEvalTC =  [stc| stc@Strong_EvalTC_Flag <- pargs] /= []

	noCaching    :: Bool
	noCaching    =  [nc | nc@NoCache_Flag        <- pargs] /= []


-----------------
--	"error2":  --
-----------------
--error2 :: String -> String -> a
--error2 =  errorM modName

